function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

import { useContext, useMemo, useRef, useState } from 'react';
import createQueryCacheId from './create-query-cache-id';
import { registerItemToFetchConnector, uncacheQueries } from './fetch-connector';
import fetchConnectorContext from './FetchConnectorContext';
import { FetchSSRManagerContext } from './get-markup-from-tree';
import actHack from './internal/actHack';

var incrementResponseId = function incrementResponseId(x) {
  return x + 1;
};

function getQueryResult(fetchConnector, queryId, options) {
  if (options && options.skip) {
    return {
      loading: false,
      error: undefined,
      data: undefined
    };
  }

  var cacheItem = fetchConnector.cache.state[queryId];
  return {
    loading: Boolean(cacheItem.promise),
    error: cacheItem.error,
    data: cacheItem.payload
  };
}
/**
 *
 * @param uri URI to be fetched from
 * @param options It's good practice to memoize this param
 */


export default function useFetch(uri, options) {
  var _ref = options || {},
      _ref$stackSize = _ref.stackSize,
      stackSize = _ref$stackSize === void 0 ? 3 : _ref$stackSize;

  var fetchConnector = useContext(fetchConnectorContext);
  var ssrManager = useContext(FetchSSRManagerContext);
  var queryId = useMemo(function () {
    return createQueryCacheId(uri, options);
  }, [uri, options]);
  var queriesIdsRef = useRef([queryId]);
  var localStack = useRef([queriesIdsRef.current]);

  if (!fetchConnector) {
    throw new Error('useFetch should not be used outside <FetchConnectorContext.Provider />');
  }

  var queryIdChanged = false;

  if (queryId !== queriesIdsRef.current[0]) {
    queryIdChanged = true;
    queriesIdsRef.current = [queryId];
    var identicalMainQueryIdsListIndex = -1;
    var identicalMainQueryIdsList = localStack.current.find(function (queriesIds, index) {
      if (queriesIds[0] === queryId) {
        identicalMainQueryIdsListIndex = index;
        return true;
      }

      return false;
    });

    if (identicalMainQueryIdsList) {
      localStack.current.splice(identicalMainQueryIdsListIndex, 1);
      localStack.current.push(identicalMainQueryIdsList);
      queriesIdsRef.current = identicalMainQueryIdsList;
    } else {
      localStack.current.push(queriesIdsRef.current);
    }

    if (localStack.current.length > stackSize) {
      var poppedQueriesIds = localStack.current[0];
      localStack.current = localStack.current.slice(localStack.current.length - stackSize);
      uncacheQueries(fetchConnector, poppedQueriesIds);
    }
  }

  useMemo(function () {
    if (options && options.skip) {
      return;
    }

    var promise = registerItemToFetchConnector(fetchConnector, queryId, uri, options);

    var handle = function handle(payload, isError) {
      // Prevent re-render when key changed before promise was resolved
      if (queryId !== queriesIdsRef.current[0]) {
        return;
      }

      fetchConnector.cache.state[queryId] = isError ? {
        error: payload
      } : {
        payload: payload
      };
      var cacheItem = fetchConnector.cache.state[queryId]; // We use actHack to supress react-test-renderer warning

      actHack(function () {
        resultRef.current = {
          loading: false,
          error: cacheItem.error,
          data: cacheItem.payload
        };
        setResponseId(incrementResponseId);
      });
    };

    if (promise) {
      promise.then(handle)["catch"](function (error) {
        return handle(error, true);
      });

      if (ssrManager && (!options || options.ssr !== false)) {
        ssrManager.register(promise);
      }
    }
  }, [queryId, options, options && options.skip]); // Use response id to re-render component when something changes

  var _useState = useState(0),
      setResponseId = _useState[1];

  var syncedResult = getQueryResult(fetchConnector, queryId, options);
  var resultRef = useRef(syncedResult);

  if (queryIdChanged) {
    resultRef.current = syncedResult;
  }

  var currentResult = useMemo(function () {
    var helpers = {
      fetchMore: function fetchMore(fn) {
        var plan = fn(resultRef.current);

        if (plan && plan.uri) {
          var nextQueryId = createQueryCacheId(plan.uri, plan);
          var promise = registerItemToFetchConnector(fetchConnector, nextQueryId, plan.uri, plan);
          queriesIdsRef.current.push(nextQueryId);

          if (promise) {
            // We use actHack to supress react-test-renderer warning
            actHack(function () {
              resultRef.current = _extends({}, resultRef.current, {
                loading: true
              });
              setResponseId(incrementResponseId);
            });
          }

          var handle = function handle(payload, isError) {
            if (payload) {
              fetchConnector.cache.state[nextQueryId] = isError ? {
                error: payload
              } : {
                payload: payload
              };
            } // Prevent re-render when key changed before promise was resolved


            if (queryId !== queriesIdsRef.current[0]) {
              return;
            }

            var cacheItem = fetchConnector.cache.state[nextQueryId]; // We use actHack to supress react-test-renderer warning

            actHack(function () {
              resultRef.current = {
                loading: false,
                error: cacheItem.error,
                data: cacheItem.error ? resultRef.current.data : plan.updateData(cacheItem.payload, resultRef.current.data)
              };
              setResponseId(incrementResponseId);
            });
          };

          if (promise) {
            return promise.then(handle)["catch"](function (error) {
              return handle(error, true);
            });
          } else {
            handle();
          }
        }
      }
    };
    return _extends({}, helpers, resultRef.current);
  }, [queryId, resultRef.current]);
  return currentResult;
}